from os.path import expanduser
import os
import datetime

import mpt.config
from mpt.config import Config


MPT_PATH = os.path.dirname(os.path.realpath(__file__))
MPT_BIN = os.path.join(mpt.config.DEFAULT_MOBILE_FOLDER, 'bin')
PENTEST_FOLDER = 'pentest-' + datetime.datetime.today().strftime('%Y-%m-%d') # pentest-YYYY-MM-DD
APP_FOLDER = "app"
BACKUP_FOLDER = "backup"
SCREENSHOT_FOLDER = "screenshots"
SOURCE_FOLDER = "source"
BURP_FOLDER = "burp"
FRIDA_BIN = "frida-server"
TEMP_DIR = "/tmp/local-mpt/"
BROWSER = 'chromium'
DECOMPILER='jadx' # jd-gui, luyten
DEFAULT_TERMINAL='gnome-terminal'
conf = Config()
# Default installation for is set in Config() use conf.load_config('install-dir')

# note
# sh does not support source command and it was replaces with .
# "The . is POSIX-compliant and also works in /bin/sh."

ANDROID_TOOLS = {
    # available patameters
    # bin
    #   - variable BIN should be absolute path in format <forder>/<path-to/bin>
    # dir
    #   - use DIR variable if after download the tool extracts the binary subdirectory like folder/folder/bin
    #     required if BIN starts with command instead of /
    #     will be created before download starts
    # pre
    #   - pre install instractions
    # post
    #  - post install instractions
    # bin_info
    #  - log.info(<bin_log>) before executing command
    # download_dir
    #  - define directory to download and decompress files
    # requirement_checks
    #  - define dependency-checks for the application, the check should return true or false, multiple checks are split by semicolon
    #    e.g. python -m poetry -V    will check if poetry is installed
    #    which git will check if poetry is installed
    # bin_global
    #  - takes a python dictinary for setting symbolic links after installation to link them to MPT_BIN
    #  - {link_name: 'location/to/bin/from/dir' }  the path should be relative to package {dir} folder


    # template
    #'jadx': {
    #   'url': 'https://github.com/skylot/jadx/releases/download/v1.4.7/jadx-1.4.7.zip',
    #   'info': '',
    #   'bin': os.path.join(MOBILE_FOLDER, 'jadx/bin/jadx-gui'),
    #   'bin_info': 'Open MobSF in browser: http://127.0.0.1:8000\n Use ctrl+C to close application',
    #   'dir': 'jadx',
    #   'download_dir': 'jadx'
    #   'install': 'http',
    #   'pre': 'mkdir jadx'
    #   'post': 'mv jadx-1.4.7 jadx'
    #   'requirement_checks': 'which hostapd; which dnsmasq'
    #   'bin_global': {'jadx': 'bin/jadx', 'jadx-gui': 'bin/jadx-gui'}
    #},

    'MobSF': {
        # check updates: 'https://github.com/MobSF/Mobile-Security-Framework-MobSF'
        # fix dependency in mobSF (remove if not required) ->  sed -i \'s/packaging = ">=21\.3,<22\.0"/packaging = ">=24\.2"/\' pyproject.toml
        'url': 'https://github.com/MobSF/Mobile-Security-Framework-MobSF/archive/refs/tags/v4.1.3.zip',
        'info': 'Mobile Security Framework (MobSF)',
        'bin': 'cd {}; python -m venv venv; . ./venv/bin/activate; . ./run.sh'.format(os.path.join(conf.load_config('install-dir'), 'MobSF')),
        'bin_info': 'Open MobSF in browser: http://127.0.0.1:8000 (Press CTRL+C to quit)',
        'dir': 'MobSF',
        'install': 'http',
        'post': 'mv Mobile-Security-Framework-MobSF-4.1.3 MobSF; cd MobSF; python -m venv venv; . ./venv/bin/activate; pip install poetry; sed -i \'s/packaging = ">=21\.3,<22\.0"/packaging = ">=24\.2"/\' pyproject.toml; . ./setup.sh',
    },

    # RMS installation old version
    # 'RMS': {
    #    'url': 'https://github.com/m0bilesecurity/RMS-Runtime-Mobile-Security',
    #    'info': 'Runtime Mobile Security (RMS)',
    #    'dir': 'RMS',
    #    'bin': 'node ' + os.path.join(conf.load_config('install-dir') + 'RMS/rms.js'),
    #    'bin_info': 'Running on http://127.0.0.1:5491/ (Press CTRL+C to quit)',
    #    'install': 'git',
    #    'post': 'mv RMS-Runtime-Mobile-Security RMS; cd RMS; npm install; npm run compile',
    #    'requirement_checks': 'npm -v'
    #},

    'RMS': {
        # check 'https://github.com/m0bilesecurity/RMS-Runtime-Mobile-Security',
        #
        # HOW TO SELECT A RIGHT NODEJS VERSION ??
        #   https://github.com/ChiChou/Grapefruit/wiki/How-do-I-decide-which-version-of-nodejs-to-use%3F
        #
        #   You need to find the right node version. To check which nodejs is supported, you can refer to these two pages:
        #       Releases frida -> https://github.com/frida/frida/releases
        #       Previous Releases Node.js -> https://nodejs.org/en/about/previous-releases
        #   example:
        #       last available frida node version for linux: frida-v16.5.9-node-v127-linux-x64.tar.gz   ==> v127
        #       Based on release node page: Module Version v127 ==> Node.js Version => v22.12.0
        #       we need to install v22.12.0
        #       put the correct version in post command "... nodeenv --python-virtualenv --node 22.12.0 ..."
        #
        'info': 'Runtime Mobile Security (RMS)',
        'dir': 'RMS',
        'bin': 'cd {}; . ./venv/bin/activate; rms'.format(os.path.join(conf.load_config('install-dir'), 'RMS')),
        'bin_info': 'Running on http://127.0.0.1:5491/ (Press CTRL+C to quit)',
        'install': 'local',
        # install nodejs locally within the python virtual environment
        'post': 'cd RMS; python -m venv venv; . ./venv/bin/activate; pip install nodeenv; nodeenv --python-virtualenv --node 22.12.0; npm install -g rms-runtime-mobile-security',
    },
    'objection': {
        # check: 'https://github.com/sensepost/objection',
        'info': 'Runtime Mobile Exploration Toolkit',
        'dir': 'objection',
        # 'bin_info': 'Please run "frida-ps -U" to find an app and start objection with the following command:\nobjection --gadget "<APP-NAME>" explore',
        'bin': 'cd {}; . ./venv/bin/activate; objection'.format(os.path.join(conf.load_config('install-dir'), 'objection')),
        'install': 'local',
        # 'post': 'cd objection; python -m venv venv; source ./venv/bin/activate; pip install --upgrade setuptools; pip install -U objection'
        'post': 'cd objection; python -m venv venv; . ./venv/bin/activate; pip install --upgrade setuptools; pip install -U objection'
    },
    'jadx': {
        # check updates: 'https://github.com/skylot/jadx/releases'
        'url': 'https://github.com/skylot/jadx/releases/download/v1.5.1/jadx-1.5.1.zip',
        'info': 'Dex to Java decompiler',
        'bin': os.path.join(conf.load_config('install-dir'), 'jadx/bin/jadx-gui'),
        'dir': 'jadx',
        'download_dir': 'jadx',
        'install': 'http',
        'pre': 'mkdir jadx',
        'bin_global': {'jadx': 'bin/jadx', 'jadx-gui': 'bin/jadx-gui'}
    },
    'jd-gui': {
        # check updates: 'https://github.com/java-decompiler/jd-gui/releases'
        'url': 'https://github.com/java-decompiler/jd-gui/releases/download/v1.6.6/jd-gui-1.6.6.jar',
        'info': 'Java Decompiler, dex2jar required',
        'bin': os.path.join(conf.load_config('install-dir'), 'jd-gui/jd-gui.jar'),
        'dir': 'jd-gui',
        'install': 'http',
    },
    'luyten': {
        # check updates https://github.com/deathmarine/Luyten/releases
        'url': 'https://github.com/deathmarine/Luyten/releases/download/v0.5.4_Rebuilt_with_Latest_depenencies/luyten-0.5.4.jar',
        'info': 'Java Decompiler Gui for Procyon',
        'bin': os.path.join(conf.load_config('install-dir'), 'luyten/luyten.jar'),
        'dir': 'luyten',
        'install': 'http'
    },
    'sqlitestudio': {
        # check updates : https://github.com/pawelsalawa/sqlitestudio/releases
        'url': 'https://github.com/pawelsalawa/sqlitestudio/releases/download/3.4.11/sqlitestudio-3.4.11.tar.xz',
        'info': 'Multi-platform SQLite database manager',
        'dir': 'SQLiteStudio',
        'bin': os.path.join(conf.load_config('install-dir'), 'SQLiteStudio/sqlitestudio'),
        'install': 'http',
        'bin_global': {'sqlitestudio': 'sqlitestudio', 'sqlitestudiocli': 'sqlitestudiocli'}
    },
    'pidcat': {
        'url': 'https://github.com/JakeWharton/pidcat',
        'info': 'excellent logcat color script',
        'bin': os.path.join(conf.load_config('install-dir') + 'pidcat/pidcat.py'),
        'dir': 'pidcat',
        'install': 'git'
    },
    'pidcat-ex': {
        'url': 'https://github.com/healthluck/pidcat-ex.git',
        'info': 'PID Cat (extended version)',
        'bin': os.path.join(conf.load_config('install-dir') + 'pidcat-ex/pidcat-ex.py'),
        'dir': 'pidcat-ex',
        'install': 'git'
    },
    'adus': {
        'url': 'https://github.com/coreb1t/adus',
        'info': 'Bash script to dump, build and sign apk',
        'bin': os.path.join(conf.load_config('install-dir') + 'adus/adus.sh'),
        'dir': 'adus',
        'install': 'git'
    },
    'fridump': {
        # check https://github.com/rootbsd/fridump3 (updated version)
        # https://github.com/Nightbringer21/fridump (obsolete version)
        'url': 'https://github.com/rootbsd/fridump3',
        'info': 'Memory dumping tool using frida',
        'bin': f'cd {os.path.join(conf.load_config('install-dir'),'fridump')}; . ./venv/bin/activate; python fridump3.py',
        'dir': 'fridump',
        'install': 'git',
        'post': 'cd fridump; python -m venv venv; . ./venv/bin/activate; pip install --upgrade frida-tools; chmod +x fridump3.py'
    },
    'adb': {
        'url': 'https://dl.google.com/android/repository/platform-tools-latest-linux.zip',
        'info': 'Android Debug Bridge (adb)',
        'dir': 'platform-tools',
        'bin': os.path.join(conf.load_config('install-dir'), 'platform-tools/adb'),
        'install': 'http',
        'bin_global': {'adb': 'adb', 'fastboot': 'fastboot', 'sqlite3': 'sqlite3'}
    },
    'aapt': {
        # check version and replace r34 with the latest one: https://developer.android.com/tools/releases/build-tools
        # URL example version + RC: 'https://dl.google.com/android/repository/build-tools_r34-rc1-linux.zip',
        'url': 'https://dl.google.com/android/repository/build-tools_r34-linux.zip', # android 14
        'info': 'Android Asset Packaging Tool',
        'bin': os.path.join(conf.load_config('install-dir'), 'build-tools/aapt'),
        'dir': 'build-tools',
        'install': 'http',
        'post': 'mv android-14 build-tools',
        #' cp -r android-14/lib64 build-tools/lib; cp -r android-14/lib64 build-tools/lib; cp android-14/aapt build-tools/aapt; cp android-14/aapt2 build-tools/aapt2; cp android-14/apksigner build-tools/apksigner ;rm -rf android-14',
        'bin_global': {'aapt': 'aapt', 'aapt2': 'aapt2', 'apksigner': 'apksigner', 'dexdump': 'dexdump'}
    },
    'abe': {
        # check updates: https://github.com/nelenkov/android-backup-extractor/releases
        'url': 'https://github.com/nelenkov/android-backup-extractor/releases/download/latest/abe-0059753.jar', # 2024-11
        'info': 'Android backup extractor, android:allowBackup="true" required',
        'bin': os.path.join(conf.load_config('install-dir'), 'abe/abe.jar'),
        'dir': 'abe',
        'install': 'http'
    },
    'signapk': {
        'url': 'https://github.com/appium-boneyard/sign',
        'info': 'sign an apk with the Android test certificate',
        'bin': os.path.join(conf.load_config('install-dir'),'sign/dist/signapk.jar'), # not tested
        'dir': 'sign',
        'install': 'git'
    },
    'apktool': {
        # check http://ibotpeaches.github.io/Apktool/ and https://apktool.org/blog
        'url': 'https://bitbucket.org/iBotPeaches/apktool/downloads/apktool_2.10.0.jar',
        'info': 'A tool for reverse engineering Android apk files',
        'bin': os.path.join(conf.load_config('install-dir'), 'apktool/apktool.jar'),
        'dir': 'apktool',
        'install': 'http'
    },
    'dex2jar': {
        # check https://github.com/pxb1988/dex2jar/
        'url': 'https://github.com/pxb1988/dex2jar/releases/download/v2.4/dex-tools-v2.4.zip',
        'info': 'Convert the Dalvik Executable (.dex) file to jar',
        'bin': os.path.join(conf.load_config('install-dir'), 'dex2jar/d2j-dex2jar.sh'),
        'dir': 'dex2jar',
        'install': 'http',
        'post': 'mv dex-tools-v2.4 dex2jar',
        'bin_global': {
            'd2j-apk-sign.sh': 'd2j-apk-sign.sh', 'd2j-asm-verify.sh': 'd2j-asm-verify.sh', 'd2j-baksmali.sh': 'd2j-baksmali.sh',
            'd2j-class-version-switch.sh': 'd2j-class-version-switch.sh', 'd2j-decrypt-string.sh': 'd2j-decrypt-string.sh', 'd2j-dex2jar.sh': 'd2j-dex2jar.sh',
            'd2j-dex2smali.sh': 'd2j-dex2smali.sh', 'd2j-dex-recompute-checksum.sh': 'd2j-dex-recompute-checksum.sh', 'd2j-dex-weaver.sh': 'd2j-dex-weaver.sh',
            'd2j_invoke.sh': 'd2j_invoke.sh', 'd2j-jar2dex.sh': 'd2j-jar2dex.sh', 'd2j-jar2jasmin.sh': 'd2j-jar2jasmin.sh',
            'd2j-jar-access.sh': 'd2j-jar-access.sh', 'd2j-jar-weaver.sh': 'd2j-jar-weaver.sh', 'd2j-jasmin2jar.sh': 'd2j-jasmin2jar.sh',
            'd2j-smali.sh': 'd2j-smali.sh','d2j-std-apk.sh': 'd2j-std-apk.sh'
        }
    },
    'janus': {
        'url': 'https://github.com/ppapadatis/python-janus-vulnerability-scan',
        'info': 'scans an APK and an Android device for CVE-2017–13156',
        'bin': 'cd {}; . ./venv/bin/activate; python janus.py'.format(os.path.join(conf.load_config('install-dir'), 'python-janus-vulnerability-scan')),
        'dir': 'python-janus-vulnerability-scan',
        'install': 'git',
        'post': 'cd python-janus-vulnerability-scan; python -m venv venv; . ./venv/bin/activate; pip install -r requirements.txt'
    },
    'linux-router': {
        'url': 'https://github.com/garywill/linux-router',
        'info': 'Set Linux as router in one command. Able to provide Internet, or create WiFi hotspot',
        'bin': 'cd {}; sudo ./lnxrouter'.format(os.path.join(conf.load_config('install-dir'), 'linux-router')),
        'dir': 'linux-router',
        'install': 'git',
        'requirement_checks': 'which hostapd; which dnsmasq'
    },
    'kitty': {
        # check https://sw.kovidgoyal.net/kitty/binary/
        # requirement tool to objection execution with interactive cli
        'info': 'The fast, feature-rich, GPU based terminal emulator',
        'bin': os.path.join(conf.load_config('install-dir'), 'kitty/kitty.app/bin/kitty'),
        'dir': 'kitty',
        'install': 'local',
        'post': 'curl -L https://sw.kovidgoyal.net/kitty/installer.sh | sh /dev/stdin dest={} launch=n'.format(os.path.join(conf.load_config('install-dir'),'kitty'))
    },
    'scrcpy': {
        # check version and replace file: https://github.com/Genymobile/scrcpy
        'url': 'https://github.com/Genymobile/scrcpy/releases/download/v3.1/scrcpy-linux-x86_64-v3.1.tar.gz',
        'info': 'Application mirrors Android devices (video and audio) connected via USB',
        'bin': os.path.join(conf.load_config('install-dir'), 'scrcpy/scrcpy'),
        'dir': 'scrcpy',
        'install': 'http',
        'post': 'mv scrcpy-linux-x86_64-v3.1 scrcpy'
    }
}

ANDROID_APKS = {
    'Xposed':{
       'url': 'https://forum.xda-developers.com/attachment.php?attachmentid=4393082&d=1516301692', # link with redirect
            # https://repo.xposed.info/module/de.robv.android.xposed.installer
            # https://forum.xda-developers.com/showthread.php?t=3034811
        'apk': 'apps/XposedInstaller_3.1.5.apk',
        'pkg': 'de.robv.android.xposed.installer'
    },

    'JustTrustMe': {
        'url' :'https://github.com/Fuzion24/JustTrustMe',
        'apk': 'apps/JustTrustMe-singed.apk',
        'pkg': 'just.trust.me'

        # latest version is not available in releases https://github.com/Fuzion24/JustTrustMe/releases
        # compiled latest version with gradlew
        # git clone https://github.com/Fuzion24/JustTrustMe.git
        # export ANDROID_HOME=~/Android/Sdk; cd JustTrustMe; ./gradlew assembleRelease
    },

    'Drozer Agent': {
        'url': 'https://github.com/mwrlabs/drozer/releases/download/2.3.4/drozer-agent-2.3.4.apk',
        'apk': 'apps/drozer-agent-2.3.4.apk',
        'pkg': 'com.mwr.dz'
    },

    'Inspeckage': {
        'url': 'https://github.com/ac-pm/Inspeckage/releases/download/v2.4/app-release.apk',
        'apk': 'apps/inspackage-v2.4.apk',
        'pkg': 'mobi.acpm.inspeckage'
    },

    'RootCloak': {
        'url': 'https://github.com/devadvance/rootcloak/releases/download/v3.0-beta_20160731_2/app-release.apk',
        'apk': 'apps/RootCloak-v3.0beta.apk',
        'pkg': 'com.devadvance.rootcloak2'
    }
    #'droidmon': 'https://github.com/idanr1986/droidmon',
}



import os

from setuptools import setup

setup( 
    name='mpt', 
    version='0.7.1', 
    description='Mobile Pentest Toolkit (MPT)', 
    author='Alexander Subbotin', 
    author_email='dev@bytesnipers.com', 
    url='https://github.com/ByteSnipers/mobile-pentest-toolkit',
    packages=['mpt'], 
    install_requires=[ 
        'GitPython',
        'colorlog',
        'colorama',
        'tabulate',
        'frida-tools',
        'simple_term_menu'
    ], 
     entry_points={
        'console_scripts': [
            'mpt=mpt.console:cli'
        ],
    }
) 



import os
import random
import string
import time

import subprocess
import sys
import shutil

import requests
from git import Repo
from simple_term_menu import TerminalMenu

from tabulate import tabulate
from colorama import Fore, Back, Style
from urllib.parse import urlparse

import mpt
from mpt import settings
from mpt import logger
from mpt.config import Config

log = logger.getLogger()


def yes_no(message):
    options = ["Yes", "No"]
    terminal_menu = TerminalMenu(options, title=message)
    menu_entry_index = terminal_menu.show()

    if menu_entry_index == 0:
        return True
    else:
        return False


def check_command(command_name):
    if not shutil.which(command_name):
        log.error("Couldn't find {}".format(command_name))
        sys.exit(1)


def print_device_table():
    devices = run_command(command='adb devices -l | grep -w device')

    # You can see all available properties with this command:
    # adb shell getprop
    os_version = 'Android ' + str(run_command(command='adb shell getprop ro.build.version.release')[0]).replace("\n", '')
    api = str(run_command(command='adb shell getprop ro.build.version.sdk')[0]).replace("\n", '')

    devs = []
    for device in ''.join(devices).split('\n'):
        info = device.split()
        model = "unknown"
        if len(info) > 0:
            for i in info:
                if i.find('product:') >= 0:
                    product = i.split(':')[1]
                if i.find('model:') >= 0:
                    model = i.split(':')[1]

            devs.append([info[0], product, model, os_version, api])
    log.info('ADB attached devices:')
    print(tabulate(devs, headers=['ID', 'Product', 'Model', 'OS', 'API Level']))


def check_adb_device():
    check_command(command_name='adb')
    devices = run_command(command='adb devices -l | grep -w device')

    if not devices:
        log.warn('Trying to restart adb service')
        restart_adb_server()

        devices = run_command(command='adb devices -l | grep -w device')
        if not devices:
            log.error('No connected devices found')
            sys.exit()

    devices_string = ''.join(devices)
    # fix adb no permissions issue
    if devices_string.find('no permissions') >= 0:
        log.warn('adb does not have permissions to communicate with device')
        run_command(command='adb devices -l | grep -w device', print_output=True)
        log.warn('Restarting adb')
        restart_adb_server()
        devices = run_command(command='adb devices -l | grep -w device')

    if len(devices) > 1:
        log.error('Currently only single attached device is supported. Number of connected devices: {}'.format(len(devices)))
        log.debug('Connected devices:')
        log.debug(devices_string)
        sys.exit(1)

    print_device_table()


def restart_adb_server():
    run_as(user='user')

    return_code = run_command('adb --version | grep -i version', returncode=True)

    if return_code != 0:
        conf = Config()
        install_dir = conf.load_config('install-dir')
        log.error('adb is not installed. Please install package android-sdk-platform-tools '
                  'or add {} to your $PATH variable'.format(os.path.join(install_dir, "platform-tools")))
        sys.exit(return_code)

    adb_bin = settings.ANDROID_TOOLS['adb']['bin']

    log.info(f'Executing: {adb_bin} start-server')
    run_command(f'sudo {adb_bin} kill-server', return_output=True)
    run_command('sudo killall adb')
    run_command(f'sudo {adb_bin} start-server')
    output = run_command(f"{adb_bin} devices -l", return_output=True)

    if not (''.join(output)).find('product') > 0:
        log.error('Restarting adb server failed')
        log.error(f"Run the following commands manually: sudo {adb_bin} kill-server; sudo killall adb; sudo {adb_bin} start-server")


def run_command(command, shell=True, return_output=True, print_output=False, universal_newlines=True,
                returncode=False):
    output = error = []
    try:
        log.debug(f'Executing command: {command}')
        process = subprocess.Popen(command, shell=shell, stdout=subprocess.PIPE,
                                   universal_newlines=universal_newlines)

        while True:
            nextline = process.stdout.readline()
            if nextline == '' and process.poll() is not None:
                break
            if print_output:
                sys.stdout.write(nextline)
            output.append(nextline)
            sys.stdout.flush()

        if returncode:
            return process.returncode

        if output and return_output:
            return output

    except Exception as e:
        log.exception(str(e))
        if error:
            log.error(error)
        sys.exit(1)


def run_interactive_command(command):
    log.debug('Executing interactive command: {}'.format(command))

    cmd = command.split()
    code = os.spawnvpe(os.P_WAIT, cmd[0], cmd, os.environ)
    if code == 127:
        log.error('{0}: command not found'.format(cmd[0]))


def app_installed(app_name):
    app_found = run_command(
        command="""adb shell 'pm list packages -f' | sed -e 's/.*=//' | sort | grep {}""".format(app_name))
    if not app_found:
        log.error('Application \"{}\" is not installed on the device'.format(app_name))
        sys.exit()


def run_as(user):
    if user == 'user':
        if os.getuid() == 0:
            log.error('Run {} as user'.format(__name__))
            sys.exit(1)
    if user == 'root':
        if os.getuid() != 0:
            log.error('Run {} as root'.format(__name__))
            sys.exit(1)


# install tools via command line from OS
# local tool installation using language specific package managers (python pip or nodejs npm)
# commands will be executed from post variable
def install_local(package):

    # check if dir variable is set
    if 'dir' not in settings.ANDROID_TOOLS[package].keys():
        log.error("variable \'dir\' for tool {} is not set".format(package))
        sys.exit(1)

    conf = Config()
    install_dir = conf.load_config('install-dir')
    tool_dir = os.path.join(install_dir, settings.ANDROID_TOOLS[package]['dir'])

    # supports only installation via post command
    if settings.ANDROID_TOOLS[package]['install'] == 'local':

        if not os.path.exists(tool_dir):
            run_command(command='mkdir {}'.format(tool_dir), print_output=True)

            # execute post instructions
            if 'post' in settings.ANDROID_TOOLS[package]:
                run_command(f'cd {install_dir}; {settings.ANDROID_TOOLS[package]['post']}', print_output=True)
                log.info('{} successfully installed'.format(package))
        else:
            log.warn('Binary {} already exists. Skip installation.'.format(tool_dir))


def install_git(package):

    # supports only git clone
    if settings.ANDROID_TOOLS[package]['install'] == 'git':
        git_repo = settings.ANDROID_TOOLS[package]['url']
        conf = Config()
        install_dir = conf.load_config('install-dir')

        clone_path = os.path.join(install_dir, settings.ANDROID_TOOLS[package]['dir'])

        if os.path.exists(clone_path):
            log.warn('Folder {} already exists. Skip installation.'.format(clone_path))
        else:

            # execute pre instructions
            if 'pre' in settings.ANDROID_TOOLS[package]:
                run_command(
                    command='cd {0}; {1}'.format(install_dir, settings.ANDROID_TOOLS[package]['pre']), print_output=True)
            log.info('Cloning repo: {}'.format(git_repo))
            Repo.clone_from(url=git_repo, to_path=clone_path)
            time.sleep(3)

            # execute post instructions
            if 'post' in settings.ANDROID_TOOLS[package]:
                run_command(command='cd {0}; {1}'.format(install_dir, settings.ANDROID_TOOLS[package]['post']), print_output=True)

            log.info('{} successfully installed'.format(package))


# download files via HTTP
def install_http(package):

    # check if dir variable is set
    if 'dir' not in settings.ANDROID_TOOLS[package].keys():
        log.error("variable \'dir\' for tool {} is not set".format(package))
        sys.exit(1)

    conf = Config()
    install_dir = conf.load_config('install-dir')
    tool_path = settings.ANDROID_TOOLS[package]['bin']
    tool_url = settings.ANDROID_TOOLS[package]['url']
    tool_dir = os.path.join(install_dir, settings.ANDROID_TOOLS[package]['dir'])

    if tool_url.endswith('zip') or tool_url.endswith('tar.xz') or tool_url.endswith('tar.gz'):
        install_http_archive(package)
    else:

        # supports only download single files via direct http link
        if settings.ANDROID_TOOLS[package]['install'] == 'http':

            if not os.path.exists(tool_dir):

                # execute pre instructions
                if 'pre' in settings.ANDROID_TOOLS[package]:
                    run_command(
                        command='cd {0}; {1}'.format(install_dir, settings.ANDROID_TOOLS[package]['pre']),
                        print_output=True)

                run_command(command='mkdir {}'.format(tool_dir), print_output=True)
                log.info('Downloading {} from {} '.format(package, tool_url))
                run_command(command='wget {} -O {} -q'.format(tool_url, tool_path), print_output=True)

                # execute post instructions
                if 'post' in settings.ANDROID_TOOLS[package]:
                    run_command(
                        command='cd {0}; {1}'.format(install_dir, settings.ANDROID_TOOLS[package]['post']), print_output=True)
                log.info('{} successfully installed'.format(package))
            else:
                log.warn('Binary {} already exists. Skip installation.'.format(tool_dir))


# download and extract zip and tar.xz files
def install_http_archive(package):

    conf = Config()
    install_dir = conf.load_config('install-dir')
    tool_url = settings.ANDROID_TOOLS[package]['url']
    tool_dir = os.path.join(install_dir, settings.ANDROID_TOOLS[package]['dir'])
    temp_archive_filename = ""
    download_dir = install_dir

    if os.path.exists(tool_dir):
        log.warn('Folder {} already exists. Skip installation.'.format(tool_dir))

    else:
        if 'download_dir' in settings.ANDROID_TOOLS[package]:
            download_dir = os.path.join(install_dir, settings.ANDROID_TOOLS[package]['download_dir'])

        # execute pre instructions
        if 'pre' in settings.ANDROID_TOOLS[package]:
            run_command(
                command='cd {0}; {1}'.format(install_dir, settings.ANDROID_TOOLS[package]['pre']), print_output=True)

        # handle zip files
        if tool_url.endswith('zip'):
            temp_archive_filename = 'download-tmp.zip'
            run_command(command='cd {0}; wget -q {1} -O {2}; unzip -q {2};'
                        .format(download_dir, tool_url, temp_archive_filename, ), print_output=True)

        # handle tar.xz files
        if tool_url.endswith('tar.xz'):
            temp_archive_filename = 'download-tmp.tar.xz'
            run_command(command=f'cd {download_dir}; wget -q {tool_url} -O {temp_archive_filename}; tar -xf {temp_archive_filename};',
                        print_output=True)

        # handle tar.gz files
        if tool_url.endswith('tar.gz'):
            temp_archive_filename = 'download-tmp.tar.gz'
            run_command(command=f'cd {download_dir}; wget -q {tool_url} -O {temp_archive_filename}; tar -xzf {temp_archive_filename};',
                        print_output=True)

        # execute post instructions
        if 'post' in settings.ANDROID_TOOLS[package]:
            run_command(
                command='cd {0}; {1}'.format(install_dir, settings.ANDROID_TOOLS[package]['post']), print_output=True)

        # remove downloaded file
        if os.path.exists(os.path.join(install_dir, temp_archive_filename)):
            run_command(command=f'cd {install_dir}; rm {temp_archive_filename}', print_output=True)

        log.info('{} successfully installed'.format(package))


# not used anymore
"""
def install_app(app):

    log.info('Installing {0} app'.format(Fore.CYAN + app + Style.RESET_ALL))

    pkg = settings.ANDROID_APKS[app]['pkg']
    apk_file = os.path.join(settings.MPT_PATH,settings.ANDROID_APKS[app]['apk'])

    if not os.path.isfile(apk_file):
        log.error('File {} not found. Installation canceled.'.format(apk_file))
        sys.exit()

    app_found = run_command(command="adb shell 'pm list packages -f' | grep {}".format(pkg))

    if not app_found:
        run_command(
            command="adb install {}".format(apk_file))
    else:
        log.warn('Package {} already installed. Skip installation.'.format(pkg))
"""


def check_requirements(package):
    # check tool requirements
    if 'requirement_checks' in settings.ANDROID_TOOLS[package].keys():
        counter = len(settings.ANDROID_TOOLS[package]['requirement_checks'].split(";"))
        installed = 0

        for requirement in settings.ANDROID_TOOLS[package]['requirement_checks'].split(";"):

            if not run_command(command=requirement, return_output=True):
                log.error(f"Missing dependency: please install the dependency to be able to run the following command: {requirement}")

                terminal_menu = TerminalMenu(["Yes", "No"], title=f"Would you like to skip installation for {package}")
                menu_entry_index = terminal_menu.show()
                if menu_entry_index == 0:
                    # skip tool installation
                    return False
                else:
                    log.info("Tools installation canceled by user")
                    sys.exit()
            else:
                installed += 1

        if installed == counter:
            return True

    else:
        # continue tool installation
        return True


# set symbolic links if package provides binaries, which should be available gloobally
# binary folder is settings.MPT_BIN
def set_symbolic_links(package):

    if 'bin_global' in settings.ANDROID_TOOLS[package].keys():

        if not os.path.isdir(settings.MPT_BIN):
            log.info(f'Creating MPT bin folder "{settings.MPT_BIN}" ...')
            os.makedirs(settings.MPT_BIN)

        installation_required = False
        # check if sym links already exists
        for link in settings.ANDROID_TOOLS[package]['bin_global'].keys():
            if os.path.islink(os.path.join(settings.MPT_BIN,link)):
                log.debug(f"link exists {link}")
            else:
                installation_required = True
                break

        if (installation_required):
            log.info(f"Linking binaries to {settings.MPT_BIN}")
            for link_name, bin_location in settings.ANDROID_TOOLS[package]['bin_global'].items():
                bin = os.path.join(mpt.config.DEFAULT_MOBILE_FOLDER, settings.ANDROID_TOOLS[package]['dir'], bin_location)

                if os.path.isfile(bin):
                    # create a symbolic link to biniry
                    #   cd tools/mobile/bin
                    #   e.q. => ln -s /home/$USER/tools/MOBILE/jadx/bin/jadx jadx
                    log.info(f"Found binary: {link_name}")
                    run_command(f'cd {settings.MPT_BIN}; ln -s {bin} {link_name}', return_output=True)


def install_tools():

    conf = Config()
    install_dir = conf.load_config('install-dir')

    for package in settings.ANDROID_TOOLS:
        log.info('Installing {0} into directory: {1}'.format(Fore.CYAN + package + Style.RESET_ALL, install_dir))

        # install package, if requirements are met
        if check_requirements(package):
            if settings.ANDROID_TOOLS[package]['install'] == 'git':
                install_git(package)
            if settings.ANDROID_TOOLS[package]['install'] == 'http':
                install_http(package)
            if settings.ANDROID_TOOLS[package]['install'] == 'local':
                install_local(package)

        set_symbolic_links(package)





def check_frida_is_running():
    frida_running = run_command(command='adb shell "ps" | grep {}'.format(settings.FRIDA_BIN), return_output=True)

    if not frida_running:
        log.error('Frida server is not running. Please start server before running the script (-f option)')
        sys.exit()

    # TODO replace ps with ps -A for Android 8.0
    # sometimes the app crashes and a new frida-helper starts.
    # If multiple helpers are running at the same time, frida does not work properly.
    pids = run_command(command='adb shell "ps" | grep frida-helper', return_output=True)
    if pids and len(pids) > 1:
        log.warn("Multiple frida-helper processes running, restarting frida")
        run_frida()


def run_frida(select_version=False):
    run_as(user='user')
    check_adb_device()

    if not os.path.exists(settings.TEMP_DIR):
        os.makedirs(settings.TEMP_DIR)
        log.info("TMP folder created: " + settings.TEMP_DIR)

    frida_bin = os.path.join(settings.TEMP_DIR + settings.FRIDA_BIN)

    # determine architecture for available frida-server binary
    bin_arch = ''
    if os.path.exists(frida_bin):
        if run_command(command='file {} | grep -i arm | grep 64'.format(frida_bin)):
            bin_arch = 'arm64'
        elif run_command(command='file {} | grep -i arm | grep 32'.format(frida_bin)):
            bin_arch = 'arm'
        elif run_command(command='file {} | grep -i elf | grep 64'.format(frida_bin)):
            bin_arch = 'x86_64'
        elif run_command(command='file {} | grep -i elf | grep 32'.format(frida_bin)):
            bin_arch = 'x86'

    arch = get_device_architecture()

    if not select_version:
        # remove frida binary if it is not match on arch
        if arch != bin_arch and bin_arch and os.path.isfile(frida_bin):
            os.remove(frida_bin)
            log.info("Wrong architecture, {} removed".format(frida_bin))
    else:
        os.remove(frida_bin)

    # download frida-server binary
    if not os.path.exists(frida_bin):
        download_frida(arch, select_version)
    else:
        log.info('File {} [{}] exists, skip downloading'.format(frida_bin, arch))

    user_id = get_shell_user_id()
    log.info('adb shell is running as {}'.format(user_id))

    kill_process_by_name('frida-helper')
    kill_process_by_name('frida-server')

    # run frida
    run_command(command='adb forward tcp:27042 tcp:27042', print_output=True)
    run_command(command=f'adb push {frida_bin} /data/local/tmp/', print_output=False)
    run_command(command=f'adb shell "chmod 755 /data/local/tmp/{settings.FRIDA_BIN}"', print_output=True)
    rversion = run_command(command=f'adb shell "/data/local/tmp/{settings.FRIDA_BIN} --version"',
                           print_output=False)[0].replace('\n', '')
    lversion = run_command('frida-ps --version')[0].replace('\n', '')

    log.info('Frida version running on Android:   {}'.format(rversion))
    log.info('Frida version installed on Desktop: {}'.format(lversion))
    if rversion != lversion:
        log.warn('Frida versions do not match')

    # kill frida processes on the device TODO
    # ps -ef | grep frida | grep -v 'grep' | awk '{print $2}' | xargs kill -9 $1

    # adb shell running as root (virtual device)
    if user_id.find('uid=0(root)') >= 0:
        command = """adb shell \"/data/local/tmp/{}\" &""".format(settings.FRIDA_BIN)
    else:
        # adb shell running as user (physical device)
        command = """adb shell "su -c \"/data/local/tmp/{}\"\" &""".format(settings.FRIDA_BIN)

    log.info(
        'Executing {}adb shell /data/local/tmp/{}'.format(
                    Style.BRIGHT, settings.FRIDA_BIN + Style.RESET_ALL))
    log.debug('Run frida command: ' + command)
    subprocess.Popen(command, shell=True, stdin=None, stdout=None, stderr=None, close_fds=True)
    check_frida_is_running()


def kill_process_by_name(process_name):

    # TODO replace ps with ps -A for Android 8.0
    pids = run_command(command='adb shell "ps" | grep {}'.format(process_name), return_output=True)

    if pids:
        for pid in pids:
            pid = pid.split()[1]
            log.debug('Killing process {} [pid:{}]'.format(process_name, pid))
            # adb shell running as root (virtual device)
            user_id = get_shell_user_id()
            if user_id.find('uid=0(root)') >= 0:
                run_command(command="adb shell \"kill -9 {}\"".format(pid))
            # adb shell running as user (physical device)
            else:
                run_command(command="adb shell \"su -c kill -9 {}\"".format(pid))


def get_shell_user_id():

    user_id = run_command(command="adb shell id")
    user_id = ''.join(user_id).strip('\n').split(' ')[0]

    # start adbd as root for android emulator
    if user_id.find('uid=0(root)') < 0:
        log.debug("Trying to restart adbd as root")
        run_command(command="adb root")
        user_id = run_command(command="adb shell id")
        user_id = ''.join(user_id).strip('\n').split(' ')[0]

    return user_id


def get_frida_selected_version():
    # GitHub API URL for Frida releases
    url = 'https://api.github.com/repos/frida/frida/releases'

    # Send a GET request to the GitHub API
    response = requests.get(url)
    response.raise_for_status()  # Raise an error for bad status codes

    # Parse the JSON response
    releases = response.json()

    # Extract the tag names of the releases
    release_versions = [release['tag_name'] for release in releases]

    terminal_menu = TerminalMenu(release_versions, title=f"Select frida version you want to run on device")
    menu_entry_index = terminal_menu.show()

    return release_versions[menu_entry_index]


def download_frida(arch, select_version):
    # download frida-server binary

    frida_bin = os.path.join(settings.TEMP_DIR + settings.FRIDA_BIN)

    if not os.path.exists(frida_bin):

        if(select_version):
            frida_version = get_frida_selected_version()
            frida_link = f"https://api.github.com/repos/frida/frida/releases/tags/{frida_version}"
        else:
            frida_link = "https://api.github.com/repos/frida/frida/releases/latest"

        # get download link
        # curl -s  https://api.github.com/repos/frida/frida/releases/latest | grep browser | grep server | grep android
        down_link = run_command(command=f'curl -s {frida_link} | grep browser | grep server | grep android')
        if not down_link:
            log.error('Download frida from github failed')
            sys.exit()

        down_link = [x.split(':', 1) for x in down_link]
        # get second item and delete newline, quotes and trailing spaces
        down_link = [x[1].replace('\n', '').replace('"', '').strip() for x in down_link]
        for link in down_link:
            if '{}.'.format(arch) in link:
                down_link = link
                break

        # download file
        filename = os.path.basename(urlparse(down_link).path)
        log.info('Downloading frida {}'.format(down_link))
        run_command(command='cd {}; wget -q {} -O {}'.format(settings.TEMP_DIR, down_link, filename), print_output=True)
        run_command(command='cd {}; unxz {}'.format(settings.TEMP_DIR, filename), print_output=True)

        filename = os.path.join(settings.TEMP_DIR + os.path.splitext(filename)[0])
        log.info('Frida filename: {}'.format(filename))
        if os.path.isfile(filename):
            run_command(command='mv {} {}'.format(filename, frida_bin))
            log.info('Frida server file (renamed): {}'.format(frida_bin))
            if os.path.isfile('{}.xz'.format(filename)):
                os.remove('{}.xz'.format(filename))
        else:
            log.error('{} file not found'.format(filename))
    else:
        log.info('File {} [{}] exists, skip downloading'.format(settings.FRIDA_BIN, arch))


def get_device_architecture():
    """
    :return: ['arm64', 'arm', 'x86_64', 'x86'] or exit
    """
    arch = run_command(command="adb shell getprop | grep -w 'cpu.abi'")
    arch = arch[0].split(' ')[1].strip('\n')
    arch = arch.replace('[', '').replace(']', '')
    log.info("Detected device architecture {}".format(Style.BRIGHT + arch + Style.RESET_ALL))

    # check architecture for physical devices
    available_arch = ['arm64', 'arm', 'x86_64', 'x86']
    if arch not in available_arch:

        if arch.find('arm64') >= 0:
            arch = 'arm64'
            log.warn('Architecture changed to {}'.format(arch))
        else:
            if arch.find('arm') >= 0:
                arch = 'arm'
                log.warn('Architecture changed to {}'.format(arch))

    if not arch:
        log.error('Frida server for architecture {} not found. Download frida manually'.format(arch))
        sys.exit()
    else:
        return arch


# data for wifi username and password generation
adjectives = [
    "Silly", "Wobbly", "Sneaky", "Clumsy", "Jolly", "Grumpy", "Nifty", "Goofy", "Zany", "Loopy",
    "Bouncy", "Giggly", "Nerdy", "Cheeky", "Witty", "Dizzy", "Funky", "Sassy", "Quirky", "Chirpy"
]
animals = [
    "Penguin", "Monkey", "Panda", "Dolphin", "Sloth", "Llama", "Otter", "Meerkat", "Ferret", "Frog",
    "Kangaroo", "Squirrel", "Hedgehog", "Turtle", "Giraffe", "Elephant", "Duck", "Fox", "Bear", "Moose"
]


def generate_funny_wifi_name():

    adj = random.choice(adjectives)
    animal = random.choice(animals)
    number = random.randint(1, 99)
    return f"{adj}{animal}{number}"


def generate_wifi_password():
    words = random.choice(adjectives) + random.choice(animals)
    symbols = random.choices(string.punctuation, k=2)
    digits = random.choices(string.digits, k=4)
    all_chars = list(words) + symbols + digits
    random.shuffle(all_chars)
    return ''.join(all_chars[:16])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import argparse
import os
import re
import shutil
import subprocess
import sys
from datetime import datetime

import netifaces
from colorama import Fore, Back, Style
from simple_term_menu import TerminalMenu

from mpt import functions

__version__ = '0.7.45'

from mpt import settings, logger
from mpt.config import Config

def create_default_pentest_folder_structure(pentest_dir):
    os.makedirs(os.path.join(pentest_dir, settings.APP_FOLDER))
    os.makedirs(os.path.join(pentest_dir, settings.BACKUP_FOLDER))
    os.makedirs(os.path.join(pentest_dir, settings.SCREENSHOT_FOLDER))
    os.makedirs(os.path.join(pentest_dir, settings.SOURCE_FOLDER))
    os.makedirs(os.path.join(pentest_dir, settings.BURP_FOLDER))

def create_pentest_folder_with_absolute_path():
    pentest_path = input("Please put absolute path to pentest project folder: ")

    if not pentest_path.startswith("/"):
        log.warn("that is not absolute path, select another folder. Setup canceled")
        sys.exit()

    use_tool_dir = functions.yes_no('Would you like to use this directory \"{}\" ? '.format(pentest_path))
    if use_tool_dir:
        create_default_pentest_folder_structure(pentest_path)
        return pentest_path
    else:
        log.warn("Setup canceled")
        sys.exit()


def setup_pentest(apk):
    functions.run_as(user='user')

    apk_file = ''.join(apk)
    if os.path.isfile(apk_file):
        if not (apk_file.endswith('.apk') or apk_file.endswith('.APK')):
            log.error('File does not have required extension: apk')
            sys.exit()

        pentest_path = os.path.join(os.getcwd(), settings.PENTEST_FOLDER)

        # remove pentest folder, if exists
        if os.path.isdir(pentest_path):
            log.warn("Pentest folder {} already exists.".format(pentest_path))

            options = ["Delete folder content", "select another path", "Skip setup"]
            terminal_menu = TerminalMenu(options, title="Please select an option:")
            menu_entry_index = terminal_menu.show()

            if menu_entry_index == 0:
                shutil.rmtree(pentest_path)
                log.debug(f"Folder {pentest_path} recreated")
                create_default_pentest_folder_structure(pentest_path)
            if menu_entry_index == 1:
                pentest_path = create_pentest_folder_with_absolute_path()
            if menu_entry_index == 2:
                log.warn("Folder {} already exists. Skip setup".format(settings.PENTEST_FOLDER))
                sys.exit()

        else:

            # create a new pentest project
            option1 = "use folder \"{}\"".format(pentest_path)
            options = [option1, "select another path", "skip setup"]
            terminal_menu = TerminalMenu(options, title="Start new pentest project ...")
            menu_entry_index = terminal_menu.show()

            if menu_entry_index == 0:
                # create default folder structure
                create_default_pentest_folder_structure(pentest_path)
            if menu_entry_index == 1:
                pentest_path = create_pentest_folder_with_absolute_path()
            # Skip setup
            if menu_entry_index == 2:
                log.warn("Skip setup".format(settings.PENTEST_FOLDER))
                sys.exit()

        if not os.path.isdir(pentest_path):
            log.error("Error: folder {} could not be created".format(pentest_path))
            sys.exit()

        log.info("Folder for security assessment {} created".format(Fore.CYAN + settings.PENTEST_FOLDER + Style.RESET_ALL))

        # Replace masked characters with "", fix errors with special chars in shell
        new_apk_filename = re.sub(r'[^\w.-]', '', apk_file)
        if apk_file != new_apk_filename:
            log.warn(f"APK file renamed to {new_apk_filename}")
        app_pentest_file_location = os.path.join(settings.APP_FOLDER, os.path.basename(new_apk_filename))
        app_pentest_file = os.path.join(pentest_path, app_pentest_file_location)
        shutil.copy(apk_file, app_pentest_file)

        # update apk information

        # get package name and application label
        # aapt dump badging <path-to-apk> | grep package
        # aapt dump badging <path-to-apk> | grep -w "application-label:"
        aapt_bin = settings.ANDROID_TOOLS['aapt']['bin']
        output = functions.run_command(f"{aapt_bin} dump badging {app_pentest_file}")
        output = "".join(output)

        package_match = re.search(r"package: name='(.*?)'", output)
        application_label_match = re.search(r"application-label:'(.*?)'", output)

        package = package_match.group(1) if package_match else None
        application_label = application_label_match.group(1) if application_label_match else None

        # update configuration
        conf = Config()
        conf.update('pentest-dir', pentest_path)
        conf.update('app', app_pentest_file_location)
        conf.update('package-name', package)
        conf.update('application-label', application_label)
        conf.print()

        # app install
        install_app = functions.yes_no('Would you like to install application \"{}\" on device? '.format(package))

        if install_app:
            functions.check_adb_device()
            log.info('Installing apk file: {}'.format(apk_file))
            functions.run_command('adb install "{}"'.format(apk_file))

    else:
        log.error('File {} does not exist'.format(apk_file))


def update_mpt():
    log.info("Updating TMP from https://github.com/ByteSnipers/mobile-pentest-toolkit ...")
    output = functions.run_command('pipx install git+https://github.com/ByteSnipers/mobile-pentest-toolkit --include-deps --force', return_output=True)

    installed_version = ""
    for out in output:
        if out.find('installed package mpt') > 0:
            installed_version = out.strip()

    if installed_version:
        log.success(installed_version)
    else:
        for out in output:
            log.error(out.strip())


def list_packages(show_all_pkgs):
    functions.run_as(user='user')
    functions.check_adb_device()
    print_all = False

    if show_all_pkgs == 'all':
        print_all = True

    if print_all:
        log.info('Print all installed packages')
    else:
        log.info('Only user apps are listed. ' + Style.BRIGHT + Fore.WHITE + 'Use option \'-l all\' to display all packages.' + Style.RESET_ALL)

    packages = functions.run_command(command="""adb shell 'pm list packages -f' | cut -d ":" -f 2""",
                                     print_output=False)

    for p in sorted(packages):

        package = p.rstrip("\n\r").split(".apk=")
        package[0] = package[0] + ".apk"

        # print user packages in bold, do not show default google applications
        if package[0].find('/data/app/') >= 0 \
                and not (package[1].startswith('com.google') or package[1].startswith('com.android')):
            package[0] = Style.BRIGHT + package[0] + Style.RESET_ALL

        # print all apps including system
        if print_all:
            print(Fore.CYAN + package[1] + Style.RESET_ALL)
            print("[APP]: " + package[0])
            print("[DIR]: " + os.path.join("/data/data/", package[1]))

        # print only user apps
        else:
            # do not show default google applications
            if package[0].find('/data/app/') >= 0 \
                    and not (package[1].startswith('com.google') or package[1].startswith('com.android')):

                code = functions.run_command(
                    'adb shell dumpsys package {} | grep -i pkgFlags | grep ALLOW_BACKUP'.format(package[1]),
                    returncode=True)

                if code:
                    backup_status = "[BACKUP]: " + Fore.YELLOW + "disabled" + Style.RESET_ALL
                else:
                    backup_status = "[BACKUP]: " + Fore.GREEN + "enabled" + Style.RESET_ALL

                print(Fore.CYAN + package[1] + Style.RESET_ALL)
                print(Style.BRIGHT + "[APP]:    " + package[0] + Style.RESET_ALL)
                print(Style.BRIGHT + "[DIR]:    " + os.path.join("/data/data/", package[1]) + Style.RESET_ALL)
                print(backup_status)


def run_pidcat(package_name):
    functions.run_as(user='user')
    functions.check_adb_device()

    if not package_name:
        conf = Config()
        package_name = conf.load_config('package-name')

    functions.app_installed(package_name)

    tool_name = 'pidcat'
    width_left = 30
    pidcat_bin = settings.ANDROID_TOOLS[tool_name]['bin']

    functions.run_command(command='python3 -u  {} -w {} {}'.format(pidcat_bin, width_left, package_name), print_output=True,
                          universal_newlines=True)


def run_pidcat_ex(package_name):
    functions.run_as(user='user')
    functions.check_adb_device()

    if not package_name:
        conf = Config()
        package_name = conf.load_config('package-name')

    functions.app_installed(package_name)

    tool_name = 'pidcat-ex'
    width_left = 30
    pidcat_bin = settings.ANDROID_TOOLS[tool_name]['bin']

    functions.run_command(command='python3 -u {0} -w {1} {2} --hl \'{2}\\yellow|/data/data/\\blue|/data/user/\\blue|/data/app/\\blue|.so\\cyan|activity\\green|Exception\\bg_red\''
                          .format(pidcat_bin, width_left, package_name), print_output=True, universal_newlines=True)


def get_backup_folder(package_name):
    """
    This method generates filename like backup<number>_<package_name>_<comment>

    :param package_name:
    :return: absolute_path to the backup folder
    """
    comment = input('Enter your comment for backup (optional): ')
    log.debug("comment: {} , len = {}".format(comment, len(comment)))

    backup_dir = ""
    # find next folder index
    for index in range(1, 100):
        if len(comment) > 0:
            dir_name = 'backup{}_{}_{}'.format(index, package_name, comment)
        else:
            dir_name = 'backup{}_{}'.format(index, package_name)
        conf = Config()
        pentest_dir = conf.load_config('pentest-dir')

        # if pentest project is set
        if pentest_dir:
            backup_dir = os.path.join(pentest_dir, settings.BACKUP_FOLDER, dir_name)
            # list directories
            dirs = ''.join(os.listdir(os.path.join(pentest_dir, settings.BACKUP_FOLDER)))

        # otherwise save backup in local folder
        else:
            backup_dir = os.path.join(os.getcwd(), dir_name)
            # list directories
            dirs = ''.join(os.listdir(os.getcwd()))

        # check if backup<index> folder does not exist
        if dirs.find("backup" + str(index)) < 0:
            # backup_dir =  backup<last_index>
            log.info('Backup folder: {}'.format(backup_dir))
            break
    return backup_dir


def dump_appdata_as_backup(backup_dir, package_name):
    """
    Pull /data/data/<package_name> folder from device to local backup_dir
    :param backup_dir:
    :param package_name:
    :return:
    """

    user_id = functions.get_shell_user_id()

    # adb shell running as root (virtual device)
    if user_id.find('uid=0(root)') >= 0:
        log.info("Running on virtual device")
        commands = ["cd {}; adb pull /data/data/{}".format(backup_dir, package_name), f'cd {backup_dir}; mv {os.path.join(package_name, "*")} {backup_dir}',
                    f'cd {backup_dir}; rmdir {package_name}']

    else:
        tmp_file = "tmp_app_78923468971.tar.gz"
        log.info("Running on physical device")

        # adb shell running as user (physical device)
        commands = [f'adb shell su -c \"tar -czf /sdcard/{tmp_file} /data/data/{package_name}\"',
                    f'cd {backup_dir}; adb pull /sdcard/{tmp_file}',
                    f'adb shell su -c \"rm /sdcard/{tmp_file}\"',
                    # tar -xzvf /sdcard/backup.tar.gz -C /tmp
                    f'cd {backup_dir}; tar -xzf {tmp_file}',
                    f'cd {backup_dir}; rm {tmp_file}',
                    f'cd {backup_dir}; mv {os.path.join("data/data/", package_name, "*")} {backup_dir}',
                    f'cd {backup_dir}; rmdir {os.path.join("data/data/" + package_name)}',
                    f'cd {backup_dir}; rmdir data/data',
                    f'cd {backup_dir}; rmdir data',
                    ]

    for cmd in commands:
        functions.run_command(cmd)


def backup_app(package_name):
    functions.run_as(user='user')
    functions.check_adb_device()
    abe_bin = settings.ANDROID_TOOLS['abe']['bin']
    app_tar = "backup.tar"
    adb_backup = "backup.ab"

    if not package_name:
        conf = Config()
        package_name = conf.load_config('package-name')

    if not package_name:
        log.error("parameter \"package_name\" is required. Use " + Style.BRIGHT + "mpt -l" + Style.NORMAL + " to list packages")
        sys.exit()

    functions.app_installed(package_name)

    # check whether backup is allowed: <application android:allowBackup="true">
    code = functions.run_command(
        'adb shell dumpsys package {} | grep -i pkgFlags | grep ALLOW_BACKUP'.format(package_name), returncode=True)

    # backup disabled
    if code:
        log.info("Application {}: android:allowBackup={}. Backup is disabled".format(
            Fore.CYAN + package_name + Style.RESET_ALL, Style.BRIGHT + Fore.WHITE + "[FALSE]" + Style.RESET_ALL))
        log.info("dump application using adb pull \"{}\"".format("/data/data/" + package_name))

        backup_dir = get_backup_folder(package_name)
        os.makedirs(backup_dir)

        # dump application using adb pull
        dump_appdata_as_backup(backup_dir, package_name)

    # backup enabled
    else:
        log.info("Application {}: android:allowBackup={}. Backup is enabled"
                 "".format(Fore.CYAN + package_name + Style.RESET_ALL, Style.BRIGHT + Fore.WHITE + "[TRUE]" + Style.RESET_ALL))

        options = ["dump application using adb pull \"{}\"".format("/data/data/" + package_name),
                   "Backup application using Android backup extractor (ABE) tool"]
        terminal_menu = TerminalMenu(options, title="Please select an option:")
        menu_entry_index = terminal_menu.show()

        # dump application using adb pull
        if menu_entry_index == 0:
            backup_dir = get_backup_folder(package_name)
            os.makedirs(backup_dir)

            print(backup_dir)

            dump_appdata_as_backup(backup_dir, package_name)

        # Backup application using ABE
        if menu_entry_index == 1:

            log.info('Backing up application using Android backup extractor (ABE) {}'.format(package_name))
            backup_dir = get_backup_folder(package_name)

            print("Please enter password 0000 on the device")
            backup_dir_tmp = backup_dir + '_tmp'
            os.makedirs(backup_dir_tmp)

            backup_file = os.path.join(backup_dir_tmp, adb_backup)
            app_tar = os.path.join(backup_dir_tmp, app_tar)
            functions.run_command('adb backup -app -f "{0}" {1}'.format(backup_file, package_name))

            log.debug("tar file: " + app_tar)
            log.info('Unpacking backup file')
            functions.run_command(command='java -jar {} unpack "{}" "{}" 0000 &> /dev/null'.format(abe_bin, backup_file, app_tar),
                                  print_output=True)

            file_list = functions.run_command(command='tar -tvf "{}"'.format(app_tar), return_output=True)

            if not file_list:
                log.error("Backup file {} is empty. Maybe the application is compiled with android:allowBackup=\"false\""
                          .format(os.path.basename(app_tar)))
                shutil.rmtree(backup_dir_tmp)
                sys.exit()

            functions.run_command(command='tar xf "{}" -C "{}"'.format(app_tar, backup_dir_tmp), print_output=True)

            # move files to backup folder
            shutil.copytree(os.path.join(backup_dir_tmp + '/apps/' + package_name + '/'), backup_dir)

            files = {'db': 'databases',
                     'f': 'files',
                     'sp': 'shared_prefs',
                     'ef': 'external_storage_files',
                     'r': 'root'}
            # rename files
            for key in files.keys():
                if os.path.isdir(os.path.join(backup_dir, key)):
                    shutil.move(os.path.join(backup_dir, key), os.path.join(backup_dir, files[key]))

            shutil.rmtree(backup_dir_tmp)

    log.success("Backup successfully created")


def generate_jar_file():
    """
    :return converted jar from APP_FOLDER:
    """
    conf = Config()
    app_dir = os.path.join(conf.load_config('pentest-dir'), settings.APP_FOLDER)
    apk_file = conf.load_config('app')
    jar_file = ""
    for file in os.listdir(app_dir):
        if file.endswith('.jar'):
            jar_file = file
            log.debug(f"jar file available: {jar_file}")

    if not jar_file:
        # run command cd pentest-folder; d2j-dex2jar  test.apk
        code = functions.run_command('cd {}; {} "{}"'.format(app_dir, settings.ANDROID_TOOLS['dex2jar']['bin'],
                                                             os.path.basename(apk_file)),
                                     returncode=True, return_output=False)
        if code != 0:
            log.error('Converting dex to class files using dex2jar failed')
            exit(1)

        for file in os.listdir(app_dir):
            if file.endswith('.jar'):
                jar_file = file

        log.debug(f"jar file generated: {jar_file}")

    return jar_file


def run_decompiler(decompiler):
    functions.run_as(user='user')

    if len(decompiler) == 0:
        decompiler = settings.DECOMPILER

    if decompiler in ['jd-gui', 'jadx', 'luyten']:
        log.info("Decompiler: " + decompiler)

        conf = Config()
        if decompiler == 'jd-gui':
            app_dir = os.path.join(conf.load_config('pentest-dir'), settings.APP_FOLDER)
            jar_file = generate_jar_file()
            decompiler_bin = os.path.join(conf.load_config('install-dir'), settings.ANDROID_TOOLS['jd-gui']['bin'])
            functions.run_command('cd {}; java -jar {} "{}"'.format(app_dir, decompiler_bin, jar_file), print_output=True)

        if decompiler == 'jadx':
            apk_file = os.path.join(conf.load_config('pentest-dir'), conf.load_config('app'))
            decompiler_bin = os.path.join(conf.load_config('install-dir'), settings.ANDROID_TOOLS['jadx']['bin'])
            functions.run_command('{} "{}"'.format(decompiler_bin, apk_file), print_output=True)

        if decompiler == 'luyten':
            app_dir = os.path.join(conf.load_config('pentest-dir'), settings.APP_FOLDER)
            jar_file = generate_jar_file()
            decompiler_bin = os.path.join(conf.load_config('install-dir'), settings.ANDROID_TOOLS['luyten']['bin'])
            functions.run_command('cd {}; java -jar {} "{}"'.format(app_dir, decompiler_bin, jar_file), print_output=True)
    else:
        log.warn("Decompiler {} is unknown. Available options: jadx|jd-gui|luyten".format(decompiler))
        sys.exit(1)


# prints usage of run_tool
def print_run_tool_usage():

    print("list of available tools:")
    for package in settings.ANDROID_TOOLS:

        if 'info' in settings.ANDROID_TOOLS[package].keys():
            placeholder = ""
            if len(package) < 5:
                placeholder = "       "
            print(" * {}{} \t [ {} ]".format(package, placeholder, settings.ANDROID_TOOLS[package]['info']))
        else:
            print(" * {}".format(package))

    print("\n use: mpt -t <toolname> '<arg1> <arg2> ...' to run command")
    print("  if you pass the argument to file location please use absolute path !!!")


def run_tool_objection():
    functions.run_as(user='user')
    functions.check_adb_device()
    functions.check_frida_is_running()

    log.info("Execute the followings steps")
    log.info(f"1) Please run 'frida-ps -U' to find the app name")
    log.info(f"2) start objection with the following command: 'objection --gadget \"<APP-NAME>\" explore'")

    conf = Config()
    package = conf.load_config('package-name')
    application_label = conf.load_config('application-label')
    command = settings.ANDROID_TOOLS['objection']['bin']

    if len(application_label) > 0:

        app = f"\"{application_label}\" ({package})"
        options = [f"run objection for app {app}", "run objection"]
        terminal_menu = TerminalMenu(options, title="Please select an option:")
        menu_entry_index = terminal_menu.show()

        if menu_entry_index == 0:
            objection_command = f"objection --gadget \\\"{application_label}\\\" explore"
            command = command.rsplit(";", 1)[0] + f"; {objection_command}; exec $SHELL"
            log_info = f"Running command in {Style.BRIGHT + Fore.CYAN}kitty terminal{Style.RESET_ALL} {objection_command.replace('\\', '')}"
        if menu_entry_index == 1:
            # replace last objection mit exec $SHELL
            command = command.rsplit(";", 1)[0] + "; objection --help; exec $SHELL"
            log_info = f"Running command in {Style.BRIGHT + Fore.CYAN}kitty terminal{Style.RESET_ALL} ..."
    else:
        command = command.rsplit(";", 1)[0] + "; objection --help; exec $SHELL"
        log_info = f"Running command in {Style.BRIGHT + Fore.CYAN}kitty terminal{Style.RESET_ALL} ..."

    terminal_with_command = f"{settings.ANDROID_TOOLS['kitty']['bin']} -- bash -c \"{command}\""
    log.info(log_info)
    functions.run_command(terminal_with_command, return_output=False)


def run_tool(tool_with_args):
    """
    Accepts only two parameters tool + <arg>
    <args> multiple parameters are accepted
    :param tool_with_args:
    :return:
    """
    functions.run_as(user='user')

    if not tool_with_args:
        print_run_tool_usage()
        sys.exit()

    tool = tool_with_args[0]
    tool_args = ""
    if len(tool_with_args) > 1:
        tool_args = ' '.join(tool_with_args[1:len(tool_with_args)])

    if tool == 'list' or tool not in settings.ANDROID_TOOLS.keys():
        print_run_tool_usage()
        sys.exit()

    if tool in settings.ANDROID_TOOLS.keys():

        # check if dir variable is set
        if 'dir' not in settings.ANDROID_TOOLS[tool].keys():
            log.error("variable \'dir\' for tool {} is not set".format(tool))
            sys.exit()

        log.info("Running tool {} ...".format(Style.BRIGHT + tool + Style.RESET_ALL))
        conf = Config()
        tool_dir = os.path.join(conf.load_config('install-dir'), settings.ANDROID_TOOLS[tool]['dir'])
        if os.path.exists(tool_dir):

            try:
                log.info("Press Ctrl+C to interrupt running command")

                if 'bin_info' in settings.ANDROID_TOOLS[tool]:
                    log.info(settings.ANDROID_TOOLS[tool]['bin_info'])

                command = settings.ANDROID_TOOLS[tool]['bin']
                if command.endswith(".jar"):
                    command = 'java -jar ' + command

                # run tool + <args> (all parameters will be processed)
                if tool_args:
                    command = command + " " + tool_args

                if tool == 'objection':
                    run_tool_objection()

                # default case
                else:
                    log.debug("Command: {}".format(Style.BRIGHT + command + Style.RESET_ALL))
                    functions.run_command(command=command, return_output=True, shell=True, print_output=True)

            except KeyboardInterrupt:
                log.warn('Command execution canceled by user')
        else:
            log.error("{} not found".format(settings.ANDROID_TOOLS[tool]['bin']))
    else:
        log.error(f"{tool} tool not found")
        print_run_tool_usage()


def run_inspeckage():
    functions.run_as(user='user')
    functions.check_adb_device()
    functions.app_installed('mobi.acpm.inspeckage')

    url = 'http://127.0.0.1:8008'
    functions.run_command(command='adb forward tcp:8008 tcp:8008', print_output=True)

    # check if drozer server is running
    returncode = functions.run_command(command='curl -I -s {}'.format(url), returncode=True)

    if returncode != 0:
        log.error('Could not connect to the Inspeckage server. Please start inspeckage app on your mobile phone and chouse target application')
        sys.exit(returncode)

    functions.run_command(command=' {} {}'.format(settings.BROWSER, url), returncode=True)


def run_frida_script(package_name, frida_script):
    """
    :param package_name: Android package name
    :param frida_script: frida script file location
    :return: None
    """

    functions.run_as(user='user')
    functions.check_adb_device()

    log.debug('pwd: ' + settings.MPT_PATH)
    frida_script = os.path.join(settings.MPT_PATH, frida_script)

    if not package_name:
        conf = Config()
        package_name = conf.load_config('package-name')

    functions.app_installed(package_name)
    functions.check_frida_is_running()

    if os.path.isfile(frida_script):

        log.info("Frida script {} loaded\n".format(os.path.basename(frida_script)))
        functions.run_interactive_command(command='frida -R -f {} -l {}'.format(package_name, frida_script))
    else:
        log.error('File not found: {}'.format(frida_script))


def disable_ssl_pinning(package_name):

    log.info("Disabling SSL Pinning: " + Fore.CYAN + package_name + Style.RESET_ALL)
    frida_script = "scripts/frida/frida-sslpinning-disable.js"
    frida_script = os.path.join(settings.MPT_PATH, frida_script)

    run_frida_script(package_name, frida_script)


def disable_root_detection(package_name):

    log.info("Disabling Root Detection: " + Fore.CYAN + package_name + Style.RESET_ALL)
    frida_script = "scripts/frida/frida-bypass-root-detection.js"
    frida_script = os.path.join(settings.MPT_PATH, frida_script)

    run_frida_script(package_name, frida_script)

def take_screenshot():

    conf = Config()
    pentest_dir = conf.load_config('pentest-dir')

    # if pentest project is set
    if pentest_dir:

        screenshot_dir = os.path.join(pentest_dir, settings.SCREENSHOT_FOLDER)
        if not os.path.isdir(screenshot_dir):
            os.makedirs(screenshot_dir)

        date = datetime.today().strftime('%Y-%m-%d_%H-%M-%S')
        screenshot_file = os.path.join(screenshot_dir, f"screenshot-{date}.png")

        # adb shell screencap -p /sdcard/Download/screencap.png && adb pull /sdcard/Download/screencap.png screenshot-$(date +%Y-%m-%d_%H-%M-%S).png
        functions.run_command(command=f'adb shell screencap -p /sdcard/Download/screencap.png && adb pull /sdcard/Download/screencap.png {screenshot_file}', returncode=True)
        log.success(f"Screenshot {os.path.basename(screenshot_file)} saved to pentest folder")
    else:
        log.error("Pentest directory is not set. Run mpt --setup to create a pentest environment")



def run_fridump():
    functions.run_as(user='user')
    functions.check_adb_device()
    functions.check_frida_is_running()

    conf = Config()
    package_name = conf.load_config('package-name')
    application_label = conf.load_config('application-label')
    functions.app_installed(package_name)
    fridump_bin = settings.ANDROID_TOOLS['fridump']['bin']

    log.info(f'Dumping application memory "{application_label}" ({package_name}) ...')

    # python ~/fridump/fridump.py -U -s Chrome
    dump_output_dir = os.path.join(conf.load_config('pentest-dir'), 'dump')

    # create dump folder
    if not os.path.isdir(dump_output_dir):
        os.makedirs(dump_output_dir)
    else:
        log.warn(f"Folder {dump_output_dir} already exists")

        options = ["Yes", "No"]
        terminal_menu = TerminalMenu(options, title="Would you like to overwrite the folder")
        menu_entry_index = terminal_menu.show()

        if menu_entry_index == 0:
            shutil.rmtree(dump_output_dir)
            os.makedirs(dump_output_dir)
        if menu_entry_index == 1:
            log.info("Memory dumping skipped ..... ")
            sys.exit(0)

    functions.run_command(command=f'{fridump_bin} -u -s \'{application_label}\' -o {dump_output_dir}',
                          print_output=True,
                          universal_newlines=True)
    log.success("Application memory dump finished")
    log.info(f"Check file: {os.path.join(dump_output_dir, "strings.txt")}")

# Proxy Options
def proxy_status():
    functions.run_as(user='user')
    functions.check_adb_device()

    proxy_check = functions.run_command(command='adb shell settings get global http_proxy')[0].replace('\n', '')

    match = re.findall(r'[0-9]+(?:\.[0-9]+){3}:[0-9]+', proxy_check)
    if match:
        log.success('Proxy status: ENABLED - {}'.format(proxy_check))
    else:
        log.success('Proxy status: DISABLED')


def proxy_enable(proxy):
    functions.run_as(user='user')
    functions.check_adb_device()
    conf = Config()

    if proxy:
        match = re.findall(r'[0-9]+(?:\.[0-9]+){3}:[0-9]+', proxy)
        if not match:
            log.error(f'Required parameter for proxy missing host:port: {proxy}')
            sys.exit(1)

        proxy_host = proxy.split(':')[0]
        proxy_port = proxy.split(':')[1]
        conf.update('proxy', {'host': proxy_host, 'port': proxy_port})

    else:
        proxy_host = conf.load_config('proxy')['host']
        proxy_port = conf.load_config('proxy')['port']
        log.info(f"Load proxy configuration from settings: {proxy_host}:{proxy_port}")

    functions.run_command(command=f'adb shell settings put global http_proxy {proxy_host}:{proxy_port}', return_output=True)
    proxy_current = functions.run_command(command='adb shell settings get global http_proxy', return_output=True)[0].replace('\n', '')
    log.success('Proxy enabled: {}'.format(proxy_current))


def proxy_disable():
    functions.run_as(user='user')
    functions.check_adb_device()

    functions.run_command(command='adb shell settings put global http_proxy :0')
    log.success('Proxy status: DISABLED')


def configure_access_point():
    conf = Config()

    # read interfaces
    wifi_interfaces = functions.run_command(command='iw dev | grep -i interface | grep -vi "unnamed"', return_output=True)
    internet_interface = functions.run_command(command="route | grep '^default' | grep -o '[^ ]*$'", return_output=True)
    internet_interface = internet_interface[0].strip()

    # get default gateway
    if internet_interface:
        log.info(f"Default Gateway (internet connection): {internet_interface}")
    else:
        interface_list = netifaces.interfaces()
        interface_list.remove('lo')

        terminal_menu = TerminalMenu(interface_list, title="Please select default gateway interface (internet connection)?")
        menu_index = terminal_menu.show()
        internet_interface = interface_list[menu_index]

    # get access point interface
    available_interfaces = []
    for i in wifi_interfaces:
        available_interfaces.append(i.strip().split(' ')[1])

    terminal_menu = TerminalMenu(available_interfaces, title="Please select an interface, which can be used as access point")
    menu_index = terminal_menu.show()
    ap_interface = available_interfaces[menu_index]

    ap_name = functions.generate_funny_wifi_name()
    ap_password = functions.generate_wifi_password()
    ap_ip = conf.get_default_access_point_ip()

    conf.update('access-point', {'internet-interface': internet_interface, 'ap-interface': ap_interface, 'ap-ip': ap_ip, 'name': ap_name, 'password': ap_password})
    log.info(f"* WiFi SSID: {ap_name}")
    log.info(f"* AP Interface: {ap_interface}")
    log.info(f"* Default Gateway: {internet_interface}")
    log.info(f"* IP: {ap_ip}")
    log.warn(f"Configure Burp to set a proxy listener on the IP: {ap_ip}")
    log.success('Access point configured')


def access_point():

    conf = Config()
    access_point_conf = conf.load_config('access-point')
    
    if access_point_conf:
        log.info(f"Loaded access point (AP) configuration from settings file")
        log.warn("Root privileges required to start access point (AP)")

        linux_router_bin = settings.ANDROID_TOOLS['linux-router']['bin']
        # sudo ./lnxrouter -o <WiFi-Internet> --ap <AP-WiFi> <SSID> -p <Password> --qr
        log.warn("sudo ./lnxrouter -o <WiFi-Internet> --ap <AP-WiFi> <SSID> -p <Password> --qr")
        log.info(f"AP WiFi SSID: {access_point_conf['name']}")
        log.info(f"Connect your device to {access_point_conf['name']} with password {access_point_conf['password']}")
        log.warn(f"Configure Burp to set a proxy listener on the IP: {Fore.CYAN}{access_point_conf['ap-ip']}{Style.RESET_ALL}")
        try:
            log.info("Press Ctrl+C to interrupt this script.")
            functions.run_command(command=
                                  f"{linux_router_bin} -g {access_point_conf['ap-ip']} -o {access_point_conf['internet-interface']} "
                                  f"--ap {access_point_conf['ap-interface']} {access_point_conf['name']} -p {access_point_conf['password']} --qr",
                                  print_output=True)

        except KeyboardInterrupt:
            log.warn('Canceled by user')
            log.warn('Access point deactivated')

    else:
        configure_access_point()


def print_banner():
    banner = """{}                        __    _ __                         __            __     __              ____   _ __ 
       ____ ___   ___  / /_  (_) /__     ____  ___  ____  / /____  _____/ /_   / /_____  ____  / / /__(_) /_
      / __ \`__ \/__ \/ __ \/ / / _ \   / __ \/ _ \/ __ \/ __/ _ \/ ___/ __/  / __/ __ \/ __ \/ / //_/ / __/
     / / / / / / /_/ / /_/ / / /  __/  / /_/ /  __/ / / / /_/  __(__  ) /_/  / /_/ /_/ / /_/ / / ,< / / /_  
    /_/ /_/ /_/\____/_.___/_/_/\___/  / .___/\___/_/ /_/\__/\___/____/\__/   \__/\____/\____/_/_/|_/_/\__/  
                                     /_/                 
    {}
    @bytesnipers https://bytesnipers.com
    Written by @coreb1t (Alexander Subbotin) 
    Version: {}
    """.format(Fore.GREEN, Style.RESET_ALL, __version__)
    print(banner)


def cli():

    parser = argparse.ArgumentParser(description=print_banner())

    global log
    log = logger.getLogger()
    conf = Config()

    parser.add_argument('--update', help='Update MPT to the latest version', action='store_true')

    tools = parser.add_argument_group("Tools", "Install and run pentest tools on your host")
    tools.add_argument('-tl', '--tool-list', action='store_true', help='Show all supported tools')
    tools.add_argument('-t', '--tool', metavar='tool', nargs='+',
                       help='Run selected tool with <arguments> (use option \'list\' to display all tools)')
    tools.add_argument('-i', '--install-tools', action='store_true',
                       help='Install pentesting tools on local system in a separate environment')
    tools.add_argument('-a', '--adb-run', help='Start adb server with root to avoid a lot of issues using adb', action='store_true')

    pentest = parser.add_argument_group("Pentest", "Configure and manage current pentest")
    pentest.add_argument('--setup', metavar='[APK]', nargs=1, type=str, help='Setup pentest environment')
    pentest.add_argument('--config', help='Show current pentest config', action='store_true')

    frida = parser.add_argument_group("Frida", "Run frida server and execute frida scripts")
    frida.add_argument('-f', '--frida', help='Run frida server on the device (latest version)', action='store_true')
    frida.add_argument('-fs', '--frida-select-version', help='Run frida server on the device (select frida version)', action='store_true')
    frida.add_argument('-s', '--ssl-pinning', metavar='package-name', nargs='?', type=str, const='',
                       help='Disable SSL Pinning (<package name> optional)')
    frida.add_argument('-r', '--root-detection', metavar='package-name', nargs='?', type=str, const='',
                       help='Disable Root Detection (<package name> optional)')

    app = parser.add_argument_group("Application", "Perform app related tasks")
    app.add_argument('-l', '--list-packages', metavar='all', nargs='?', type=str, const='',
                     help='Show all installed packages (use option \'all\' to display system apps)')

    app.add_argument('-p', '--pidcat', metavar='package-name', nargs='?', type=str, const='',
                     help='Show colored logcat for a specific application (<package name> optional)')
    app.add_argument('-st', '--screenshot', help='Take a screenshot from device screen', action='store_true')
    app.add_argument('-sc', '--screen-copy', help='Mirrors Android device screen connected via USB to host (scrcpy)', action='store_true')
    app.add_argument('-D', '--fridump', help='Dump application memory', action='store_true')
    app.add_argument('-b', '--backup', metavar='package-name', nargs='?', type=str, const='',
                     help='Backup an android application \n (<package name> optional)')
    app.add_argument('-d', '--decompile', metavar='decompiler', nargs='?', type=str, const='',
                     help='Start java decompiler for source code analysis (<decompiler> optional): jadx(default), jd-gui, luyten')

    proxy_wifi = parser.add_argument_group("Proxy and WiFi", "Manage proxy on device and WiFi settings locally")
    proxy_wifi.add_argument('-ps', '--proxy-status', help='Check WiFi proxy status', action='store_true')
    proxy_wifi.add_argument('-pe', '--proxy-enable', metavar='host:port', nargs='?', type=str, const='',
                            help='Set proxy for WiFi connection on your device (optional <host:port>), if not set loads proxy settings from configuration file')
    proxy_wifi.add_argument('-pd', '--proxy-disable', help='Disable WiFi proxy', action='store_true')
    proxy_wifi.add_argument('-ap', '--access-point', help='Create an Hotspot which connected to internet and can be used for Burp proxy', action='store_true')
    # obsolete options
    # parser.add_argument('-e', '--inspeckage', help='Open Inspeckage web interface', action='store_true')
    # nargs='+' accept multiple parameters like -t janus path/to/apk

    # parser.add_argument('-t', '--tool', metavar='toolname <args>', default='list', nargs='+',
    # help='Run selected tool (use option \'list\' to display all tools)')
    # parser.add_argument('-d', '--drozer', help='Run drozer application', action='store_true')

    if len(sys.argv) == 1:
        parser.print_usage()
        sys.exit()

    args = parser.parse_args()

    if args.update:
        update_mpt()

    # Tool Options
    if args.tool_list:
        print_run_tool_usage()
    if args.tool:
        run_tool(tool_with_args=args.tool)
    if args.install_tools or type(args.install_tools) is str:
        functions.install_tools()
    if args.adb_run:
        functions.restart_adb_server()

    # Pentest Options
    if args.setup or type(args.setup) is str:
        setup_pentest(apk=args.setup)  # done
    if args.config:
        conf.print()

    # Frida options
    if args.frida:
        functions.run_frida()
    if args.frida_select_version:
        functions.run_frida(select_version=True)
    if args.ssl_pinning or type(args.ssl_pinning) is str:
        disable_ssl_pinning(package_name=args.ssl_pinning)
    if args.root_detection or type(args.root_detection) is str:
        disable_root_detection(package_name=args.root_detection)

    # Application Options
    if args.list_packages or type(args.list_packages) is str:
        list_packages(show_all_pkgs=args.list_packages)
    if args.pidcat or type(args.pidcat) is str:
        # replaced pidcat mit pidcat-extended  run_pidcat(package_name=args.pidcat)
        run_pidcat_ex(package_name=args.pidcat)
    if args.screenshot:
        take_screenshot()
    if args.screen_copy:
        run_tool(["scrcpy"])
    if args.fridump:
        run_fridump()
    if args.backup or type(args.backup) is str:
        backup_app(package_name=args.backup)
    if args.decompile or type(args.decompile) is str:
        run_decompiler(decompiler=args.decompile)

    # Proxy Options
    if args.proxy_status:
        proxy_status()
    if args.proxy_enable or type(args.proxy_enable) is str:
        proxy_enable(proxy=args.proxy_enable)
    if args.proxy_disable:
        proxy_disable()
    if args.access_point:
        access_point()

    # if args.inspeckage:
    #    run_inspeckage()
    # if args.drozer:
    #    run_drozer()
    # if args.start_appmon or type(args.start_appmon) is str:
    #    start_appmon(package_name=args.start_appmon)
